#include <Arduino.h>
// #include <Wire.h>
#include <HardwareSerial.h>
#include <string.h>
#include <libmaple/usart.h>

#include "GoGoBoardLib.h"

#define gogoSerial Serial1

uint8_t cmdCode[] = {0x54, 0xFE, 0x14};
uint8_t cmdReport[] = {0x54, 0xFE};
uint16_t stmReg[4] = {1111, 2222, 3333, 4444};

GoGoBoardLib::GoGoBoardLib(void)
{
}
GoGoBoardLib::~GoGoBoardLib(void)
{
}

void GoGoBoardLib::begin(void)
{
    gogoSerial.begin(115200);
}

uint8_t GoGoBoardLib::checkSumPkt(uint8_t *pkt)
{
    uint8_t STMPkt_checksum = 0;
    int checkLen = ((int)pkt[3]) + 3;

    for (int i = 4; i < checkLen; i++)
    {
        STMPkt_checksum += pkt[i];
    }
    STMPkt_checksum &= 0xFF;
    return STMPkt_checksum;
}

void GoGoBoardLib::setSelectMotorServo(int numSelect, int mode)
{
    char tempBuffer[4];
    String num = String(numSelect);
    int len = num.length();

    for (int i = 0; i < len; i++)
    {
        int numtemp = (int(num[i])) & 0xF;
        if (mode == 0)
        {
            numberMotor[numtemp - 1] = 1;
        }
        else if (mode == 1)
        {
            numberServo[numtemp - 1] = 1;
        }
    }
}

void GoGoBoardLib::clearMotorServo(int numSelect, int mode)
{
    for (int i = 0; i < 4; i++)
    {
        if (mode == 0)
        {
            numberMotor[i] = 0;
        }
        else if (mode == 1)
        {
            numberServo[i] = 0;
        }
    }
}

void GoGoBoardLib::gogoSendPkt(uint8_t *pktCmdCode)
{
    int pktLen = ((int)pktCmdCode[3]) + 4;
    cmdCode[pktLen - 1] = checkSumPkt(pktCmdCode);
    gogoSerial.flush();
    delay(10);
    gogoSerial.write(cmdCode, pktLen);
}

// ! ========================== GoGoBoard Motor ===================
int GoGoBoardLib::setMotorForSentPkt(int *num)
{
    int sum = 0;
    for (int i = 0; i < 4; i++) // for 4 port
    {
        int temp = 0;
        if (num[i] == 1)
        {
            temp = 1 << i;
        }
        else if (num[i] == 0)
        {
            temp = 0;
        }
        sum += temp;
    }
    return sum;
}

void GoGoBoardLib::motor_Select(int number)
{
    cmdCode[3] = 0x05; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0x07; // CMD_ID
    setSelectMotorServo(number, 0);
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor));
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::motor_OnOff(char *mode)
{
    cmdCode[3] = 0x06;                                     // LENGHT
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x02;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    // * select On Off
    String tempMode = String(mode);
    tempMode.toLowerCase();
    if (tempMode == "on")
    {
        cmdCode[8] = 0x01;
    }
    else if (tempMode == "off")
    {
        cmdCode[8] = 0x00;
    }
    else
    {
        cmdCode[8] = 0x00;
    }
    gogoSendPkt(cmdCode);
    // send to esp3 USART3
}

void GoGoBoardLib::motor_Turn(char *mode)
{
    cmdCode[3] = 0x06;                                     // LENGHT
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x03;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    // * select CW : 1 , CCW : 0
    String tempMode = String(mode);
    tempMode.toLowerCase();
    if (tempMode == "cw")
    {
        cmdCode[8] = 0x01;
    }
    else if (tempMode == "ccw")
    {
        cmdCode[8] = 0x00;
    }
    else
    {
        cmdCode[8] = 0x00;
    }
    gogoSendPkt(cmdCode); // send to esp3 USART3
}

void GoGoBoardLib::motor_Reverse(void)
{
    cmdCode[3] = 0x05;                                     // LENGHT
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x04;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    gogoSendPkt(cmdCode);                                  // send to esp3 USART3
}

void GoGoBoardLib::motor_Break(void) // No Support on GoGoBoard
{
    cmdCode[3] = 0x05;                                     // LENGHT
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x05;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    gogoSendPkt(cmdCode);                                  // send to esp3 USART3
}

void GoGoBoardLib::motor_SetPower(int val)
{
    cmdCode[3] = 0x07;                                     // LENGHT
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x06;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    uint8_t valHighByte = 0;
    uint8_t valLowByte = (uint8_t(val));
    cmdCode[8] = valHighByte;
    cmdCode[9] = valLowByte;
    gogoSendPkt(cmdCode); // send to esp3 USART3
}

void GoGoBoardLib::motor_Toggle(int val)
{
    cmdCode[3] = 0x05;                                     // LENGHT
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x08;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    gogoSendPkt(cmdCode); // send to esp3 USART3
}

void GoGoBoardLib::motor_SetServo(int val)
{
    cmdCode[3] = 0x07;                                     // LENGHT + 4
    cmdCode[4] = 0x00;                                     // ENDPOINT_ID
    cmdCode[5] = 0x00;                                     // CATEGORY_ID
    cmdCode[6] = 0x09;                                     // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberMotor)); // PARAMETER || select Motor
    uint8_t valHighByte = 0;
    uint8_t valLowByte = (uint8_t(val));
    cmdCode[8] = valHighByte;
    cmdCode[9] = valLowByte;
    gogoSendPkt(cmdCode); // send to esp3 USART3
}

// ! ========================== GoGoBoard Motor [END] ===================
// * ========================== GoGoBoard Peripheral Control ===================
void GoGoBoardLib::run_LED(int status)
{
    cmdCode[3] = 0x05; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0xA;
    cmdCode[7] = uint8_t(status);
    gogoSendPkt(cmdCode); // send to esp3 USART3
}
void GoGoBoardLib::beep(void)
{
    cmdCode[3] = 0x07; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0xB;
    cmdCode[7] = 0x00;
    cmdCode[8] = 0x00;
    cmdCode[9] = 0x00;
    gogoSendPkt(cmdCode); // send to esp3 USART3
}

void GoGoBoardLib::logo_Control(int number)
{
    cmdCode[3] = 0x05; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0xD;  // CMD_ID
    cmdCode[7] = uint8_t(number);
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::servo_Select(int number)
{
    cmdCode[3] = 0x05; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0xE;  // CMD_ID
    setSelectMotorServo(number, 1);
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberServo));
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::servo_ThisWay(int val)
{
    cmdCode[3] = 0x07; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0xF;  // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberServo));
    uint8_t valHighByte = 0;
    uint8_t valLowByte = (uint8_t(val));
    cmdCode[8] = valHighByte;
    cmdCode[9] = valLowByte;
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::servo_ThatWay(int val)
{
    cmdCode[3] = 0x07; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0x10; // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberServo));
    uint8_t valHighByte = 0;
    uint8_t valLowByte = (uint8_t(val));
    cmdCode[8] = valHighByte;
    cmdCode[9] = valLowByte;
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::servo_Angle(int val)
{
    cmdCode[3] = 0x07; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0x11; // CMD_ID
    cmdCode[7] = uint8_t(setMotorForSentPkt(numberServo));
    uint8_t valHighByte = 0;
    uint8_t valLowByte = (uint8_t(val));
    cmdCode[8] = valHighByte;
    cmdCode[9] = valLowByte;
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::servo_Toggle(int servoNumber)
{
    cmdCode[3] = 0x05; // LENGHT + 4
    cmdCode[4] = 0x00; // ENDPOINT_ID
    cmdCode[5] = 0x00; // CATEGORY_ID
    cmdCode[6] = 0x12; // CMD_ID
    cmdCode[7] = uint8_t(servoNumber);
    gogoSendPkt(cmdCode);
}

void GoGoBoardLib::convertTextForSend(char *str)
{
    String text = String(str);
    for (int i = 0; i < text.length(); i++)
    {
        oledText[i] = uint8_t(text[i]);
    }
    for (int j = 7; j < 11; j++)
    {
        cmdCode[j] = oledText[j - 7];
    }
}
void GoGoBoardLib::show_Text(char *text)
{
    cmdCode[3] = 0x08;        // LENGHT + 4x
    cmdCode[4] = 0x00;        // ENDPOINT_ID
    cmdCode[5] = 0x00;        // CATEGORY_ID
    cmdCode[6] = 0x3C;        // CMD_ID
    convertTextForSend(text); // 4 byte for 4 charecter
    gogoSendPkt(cmdCode);     // + 1 byte for checksum
}

// * ========================== GoGoBoard Peripheral Control [END]===================
// !  ================ GoGoBoard System Control ========================

void GoGoBoardLib::reboot(void)
{
    cmdCode[3] = 0x04;    // LENGHT + 4x
    cmdCode[4] = 0x00;    // ENDPOINT_ID
    cmdCode[5] = 0x00;    // CATEGORY_ID
    cmdCode[6] = 0x64;    // CMD_ID
    gogoSendPkt(cmdCode); // + 1 byte for checksum
}

// !  ================ GoGoBoard System Control [END]===================
// *  ================ GoGoBoard Data-driven Toolkit ==========================

void GoGoBoardLib::config_Wifi(int ssidLen, int pwdLen, char *ssid, char *pwd)
{
    String tempSSID = String(ssid);
    String tempPWD = String(pwd);
    int beginPWD;

    cmdCode[3] = uint8_t(ssidLen + pwdLen + 6); // LENGHT + 4
    cmdCode[4] = 0x00;                          // ENDPOINT_ID
    cmdCode[5] = 0x04;                          // CATEGORY_ID
    cmdCode[6] = 0x01;                          // CMD_ID
    cmdCode[7] = uint8_t(ssidLen);
    cmdCode[8] = uint8_t(pwdLen);

    beginPWD = 9 + ssidLen;
    for (int i = 0; i < ssidLen; i++)
    {
        gblSSID[i] = uint8_t(ssid[i]);
        cmdCode[9 + i] = gblSSID[i];
    }
    for (int i = 0; i < pwdLen; i++)
    {
        gblPWD[i] = uint8_t(pwd[i]);
        cmdCode[beginPWD + i] = gblPWD[i];
    }
    gogoSendPkt(cmdCode); // + 1 byte for checksum
}

void GoGoBoardLib::readReg(int index)
{
    cmdCode[3] = 0x06; // LENGHT + 4
    cmdCode[4] = 0x03; // ENDPOINT_ID
    cmdCode[5] = 0x01; // CATEGORY_ID
    cmdCode[6] = uint8_t(index);
    uint8_t regHighByte = stmReg[index] >> 4;
    uint8_t regLowByte = stmReg[index];
    cmdCode[7] = regHighByte;
    cmdCode[8] = regLowByte;
    gogoSendPkt(cmdReport); // + 1 byte for checksum
}
